<?php
if (!defined('PLX_ROOT')) { exit; }

/**
 * Le plugin kzMoveMyDatas permet de basculer rapidement d'un dossier de données à un autre.
 *
 * Le plugin recherche à la racine du site tous les fichiers parametres.xml
 * pour pouvoir sélectionner un dossier de données dans une liste.
 * Il permet également de renommer un dossier de données existant en mettant à jour les liens
 * dans les articles, pages statiques et commentaires.
 * Il met également à jour le fichier parametres.xml pour les racines des dossiers articles, static, ....
 *
 * Il peut aussi créer un dossier de données.
 *
 * @author	J.P. Pourrez
 * @update 2021-02-16 compatible PluXml 5.8.5
 * @version	2017-12-14
 * */
class kzMoveMyDatas extends plxPlugin {

	const PARAMETRES_FILE_PATH = 'configuration/parametres.xml';
	const CONFIG_MASK = '@^(.*\bPLX_CONFIG_PATH\b.*(?:\'|"))[^/]+/@m'; # Ne pas utiliser DIRECTORY_SEPARATOR Bug ave Windows
	/*
	const STATIC_FIELDS = 'name title_htmltag meta_description ' .
		'meta_keywords group url active menu template date_creation ' .
		'date_update readable';
	* */
	# Pour un nouveau dossier, champs utilisateurs :
	const USER_FIELDS = 'login name infos password salt email lang password_token password_token_expiry';

	const XML_HEADER = '<?xml version="1.0" encoding="' . PLX_CHARSET . '"?>' . PHP_EOL;
	const BEGIN_CODE = '<?php' . PHP_EOL;
	const END_CODE = PHP_EOL . '?>';
	const TOKEN_FILE = PLX_ROOT . __CLASS__ . '.txt';

	/**
	 * Constructeur de la class kzMoveMyDatas.
	 *
	 * Régle affichage dans le menu latéral gauche.
	 * Récupère l'adresse compléte du fichier config.php.
	 * */
	public function __construct($default_lang) {

		# appel du constructeur de la classe plxPlugin (obligatoire)
		parent::__construct($default_lang);

		$this->setAdminProfil(PROFIL_ADMIN);
		# Personnalisation du menu admin
		$label = $this->getlang('FOLDER').': ' . dirname(PLX_CONFIG_PATH) . '/';
		$title = $this->getlang('TITLE');
		$this->setAdminMenu($label, 0, $title);

		/*
		if (basename($_SERVER['SCRIPT_NAME'], '.php') == 'index') {
			$this->addHook('AdminPrepend', 'AdminPrepend');
		}
		 * */

		// $this->addHook('plxAdminConstruct', 'plxAdminConstruct');

		$this->config_filename = realpath(PLX_ROOT . 'config.php');

		if(defined('PLX_ADMIN')) {
			foreach(array('plxAdminConstruct') as $hook) {
				$this->addHook($hook, $hook);
			}
		} else {
			foreach(array('Index') as $hook) {
				$this->addHook($hook, $hook);
			}
		}
	}

	/**
	 * Récupère la liste des dossiers contenant un fichier parametres.xml au niveau 2.
	 * */
	public function getDatasFolders() {

		return array_map(
			function($item) {
				return preg_replace('@^' . PLX_ROOT . '([^/]+/).*' . self::PARAMETRES_FILE_PATH . '$@', '$1', $item);
			},
			glob(PLX_ROOT . '*/' . self::PARAMETRES_FILE_PATH)
		);
	}

	/**
	 * Crée et remplit une balise <select> pour le panneau admin.php.
	 * */
	public function printDatasFoldersSelect($name) {
?>
		<select id="id_<?php echo $name ?>" name="<?php echo $name ?>">
			<option value=""><?php $this->lang('NEW_FOLDER'); ?></option>
<?php
		$currentFolder = preg_replace('@^([^/]+/).*$@', '$1', PLX_CONFIG_PATH);
		foreach($this->getDatasFolders() as $folder) {
?>
			<option value="<?= $folder ?>" <?= ($folder == $currentFolder) ? 'selected' : '' ?>><?= $folder ?></option>
<?php
		}
?>
		</select>
<?php

	}

	/**
	 * Vérifie le droit en écriture du dossier.
	 * */
	private function writableFiles($folder) {
		global $plxAdmin;

		$folders = array(PLX_CONFIG_PATH);
		foreach(explode(' ', 'articles statiques commentaires') as $k) {
			$folders[] = $plxAdmin->aConf['racine_' . $k];
		}

		$result = true;
		foreach(glob(PLX_ROOT . '{' . explode(',', $folders) . '}/*') as $filename) {
			if (!is_writable($filename)) {
				plxMsg::Error(sprintf($this->getLang('UNWRITABLE'), $filename));
				$result = false;
				break;
			}
		}
		return $result;
	}

	/**
	 * Met à jour tous les liens des attributs href et src dans un fichier.
	 * */
	private function __updateFile($filename, $params) {

		if(!is_writable($filename)) {
			plxMsg::Error(sprintf($this->getLang('UNWRITABLE_FILE'), $filename));
			return false;
		}

		$content = file_get_contents($filename);
		if(!empty($content)) {
			# enregistre la date du fichier
			$updateTime = filemtime($filename);
			if(
				file_put_contents(
					$filename,
					preg_replace(
						'#(href="|src="|CDATA\[)'.$params['oldName'].'#',
						'$1'.$params['newName'],
						$content
					)
				)
			) {
				# restaure la date du fichier
				return touch($filename, $updateTime);
			} else {
				# echec
				return false;
			}
		} else {
			return true;
		}
	}

	/**
	 * Renomme le dossier de données en mettant à jour les liens pour les attributs href et src.
	 * Egalement pour les thumbnails des catégories.
	 * */
	private function __renameDatasFolder($oldName, $newName) {
		global $plxAdmin;

		# So, we change the name of the data folder
		$params = array(
			'oldName'	=> $oldName,
			'newName'	=> $newName,
		);
		$pattern = '@^' . $oldName . '@';

		# $plxAdmin->plxGlob_arts
		$dirname = PLX_ROOT . $plxAdmin->aConf['racine_articles'];
		foreach($plxAdmin->plxGlob_arts->aFiles as $filename) {
			if (!$this->__updateFile($dirname . $filename, $params)) {
				return false;
				break;
			}
		}

		# $plxAdmin->plxGlob_coms
		$dirname = PLX_ROOT . $plxAdmin->aConf['racine_commentaires'];
		foreach($plxAdmin->plxGlob_coms->aFiles as $filename) {
			if (!$this->__updateFile($dirname . $filename, $params)) {
				return false;
				break;
			}
		}

		# plxAdmin->aCats
		$isDirty = false;
		foreach($plxAdmin->aCats as $catId=>$infos) {
			if(isset($infos['thumbnail']) and !empty($infos['thumbnail'])) {
				$plxAdmin->aCats[$catId]['thumbnail'] = preg_replace($pattern, $newName, $infos['thumbnail']);
				$isDirty = true;
			}
		}
		if($isDirty) {
			$plxAdmin->editCategories(false, true);
		}

		# plxAdmin->aStats
		$isDirty = false;
		# $genuineFields = explode(' ', self::STATIC_FIELDS);
		$dirname = PLX_ROOT . $plxAdmin->aConf['racine_statiques'];
		foreach($plxAdmin->aStats as $number=>$values) {
			$filename = $number . '.' . $values['url'] . '.php';
			if (!$this->__updateFile($dirname . $filename, $params)) {
				return false;
				break;
			}
			/*
			foreach(array_keys($values) as $k) {
				if (!in_array($k, $genuineFields)) {
					$isDirty = true;
					$plxAdmin->aStats[$number][$k] = preg_replace('#^' . $oldName . '#', $newName, $values[$k]);
				}
			}
			* */
		}
		if ($isDirty) {
			$plxAdmin->editStatiques(false, true);
		}

		# plxAdmin->aTags
		# $plxAdmin->editTags();

		# plxAdmin->aUsers
		# $plxAdmin->users($content);

		# modifier le fichier parametres.xml dans le dossier de données
		$content = array();
		foreach(array(
			'racine_articles',
			'racine_commentaires',
			'racine_statiques',
			'medias',
		) as $k) {
			$content[$k] = preg_replace($pattern, $newName, $plxAdmin->aConf[$k]);
		}
		$plxAdmin->editConfiguration($plxAdmin->aConf, $content);

		#renommer le dossier de données
		rename(PLX_ROOT . $oldName, PLX_ROOT . $newName);

		return true;
	}

	/**
	 * Post the first article and return the first category and tags.
	 * */
	private function __firstArticle($lang, $userId, $folderName) {
		$creation = date('YmdHi');
		$filename = __DIR__ . "/sample/$lang.tpl";
		if(is_file($filename)) {
			list($titleArt, $categoryArt, $tagsArt, $chapoArt, $contentArt) = explode('<<|>>', file_get_contents($filename));
			$medias = $folderName . 'medias/';
			foreach(array('alice.jpg', 'book.jpg', 'books.webp') as $img) {
				$src = __DIR__ . '/sample/medias/' . $img;
				if(file_exists($src)) {
					copy($src, PLX_ROOT . $medias . $img);
					$thumbArt = $medias . plxUtils::thumbName($img);
					plxUtils::makeThumb($src, PLX_ROOT . $medias . '.thumbs/' . $img, 48, 48);
					plxUtils::makeThumb($src, PLX_ROOT . $thumbArt, 100, 200);
				}
			}
			$thumbArt = $medias . plxUtils::thumbName('alice.jpg');
		} else {
			$titleArt = plxUtils::strRevCheck(L_DEFAULT_ARTICLE_TITLE);
			$categoryArt = 'PluXml';
			$tagsArt = 'pluxml';
			list($chapoArt, $contentArt) = explode('-----', file_get_contents(PLX_CORE . 'lib/html.article.txt'));
			$thumbArt = 'core/admin/theme/images/pluxml.png';
		}
		ob_start();
?>
<document>
	<title><![CDATA[<?= $titleArt ?>]]></title>
	<allow_com>1</allow_com>
	<template>article.php</template>
	<chapo><![CDATA[<?= $chapoArt ?>]]></chapo>
	<content><![CDATA[<?= $contentArt ?>]]></content>
	<tags><![CDATA[PluXml]]></tags>
	<meta_description></meta_description>
	<meta_keywords></meta_keywords>
	<title_htmltag></title_htmltag>
	<thumbnail><![CDATA[<?= $thumbArt ?>]]></thumbnail>
	<thumbnail_alt></thumbnail_alt>
	<thumbnail_title><?= preg_replace('@.*/([\w-]+)\.(?:jpe?g|png|gif|svg|tiff|webp)$@', ucfirst('$1'), $thumbArt) ?></thumbnail_title>
	<date_creation><?= $creation ?></date_creation>
	<date_update><?= $creation ?></date_update>
</document>
<?php
		$filename = PLX_ROOT . $folderName . '/articles/' . implode('.', array(
			'0001', # Id article
			'001', # categorie
			$userId, # auteur
			$creation, # date publication
			L_DEFAULT_ARTICLE_URL, # url
			'xml', # extension fichier
		));
		plxUtils::write(self::XML_HEADER . ob_get_clean(), $filename);
		return array($categoryArt, $tagsArt, $creation);
	}

	/**
	 * Crée un nouvel dossier pour les données avec ses fichiers de configuration.
	 * */
	private function __createDatasFolder($folderName) {
		global $plxAdmin, $lang;

		if (!is_dir(PLX_ROOT.$folderName)) {
			if (mkdir(PLX_ROOT.$folderName)) {
				loadLang(PLX_CORE . 'lang/' . $lang . '/install.php');

				# create all folders
				foreach(array(
					' ',
					'articles',
					'commentaires',
					'statiques',
					'medias/.thumbs',
					'configuration/plugins'
				) as $f) {
					mkdir(PLX_ROOT . $folderName . $f, 0755, true);
				}

				file_put_contents(PLX_ROOT . $folderName . '.htaccess', 'Options -Indexes' . PHP_EOL);

				$userId = $_SESSION['user']; // 001
				list($titleCat, $tags, $dateArt) = $this->__firstArticle($lang, $userId, $folderName);

				# configurations
				$pattern = '#\b' . dirname(PLX_CONFIG_PATH) . '/#';
				$newConfigPath = preg_replace($pattern, $folderName, PLX_CONFIG_PATH);
				foreach(array('CATEGORIES', 'STATICS', 'TAGS', 'PLUGINS', 'USERS') as $k) {
					ob_start();
					switch ($k) {
						case 'USERS':
							$user = $plxAdmin->aUsers[$userId];
?>
<document>
	<user number="<?= $userId?>" active="1" profil="0" delete="0">
<?php
							foreach(explode(' ', self::USER_FIELDS) as $field) {
								if(isset($user[$field])) {
?>
		<<?= $field ?>><![CDATA[<?= $user[$field] ?>]]></<?= $field ?>>
<?php
								}
							}
?>
	</user>
</document>
<?php
							break;
						case 'PLUGINS':
?>
<document>
	<plugin name="<?= __CLASS__ ?>" scope=""></plugin>
</document>
<?php
							break;
						case 'CATEGORIES':
						$medias = $folderName . 'medias/';
?>
<document>
	<categorie number="001" active="1" homepage="1" tri="desc" bypage="5" menu="oui" url="literature" template="categorie.php">
		<name><![CDATA[<?= $titleCat ?>]]></name>
		<description></description>
		<meta_description></meta_description>
		<meta_keywords></meta_keywords>
		<title_htmltag></title_htmltag>
		<thumbnail><?= $medias . plxUtils::thumbName('book.jpg') ?></thumbnail>
		<thumbnail_alt></thumbnail_alt>
		<thumbnail_title>Book</thumbnail_title>
	</categorie>
</document>
<?php
							break;
						case 'TAGS':
?>
<document>
	<article number="0001" date="<?= $dateArt ?>" active="1"><![CDATA[<?= $tags ?>]]></article>
</document>
<?php
							break;
						default:
							# STATIC
?>
<document>
</document>
<?php
					}
					$filename = preg_replace($pattern, $folderName, path('XMLFILE_' . $k));
					plxUtils::write(self::XML_HEADER . ob_get_clean(), $filename);
				}

				$newConfig = array(
					'title'					=> 'Lovely build by ' . __CLASS__ . ' plugin',
					'meta_description'		=> '',
					'meta_keywords'			=> '',
					'medias'				=> $folderName . 'medias/',
					'racine_articles'		=> $folderName . 'articles/',
					'racine_commentaires'	=> $folderName . 'commentaires/',
					'racine_statiques'		=> $folderName . 'statiques/',
					'urlrewriting'			=> 0, /* Important pour forcer la ré-écriture de .htaccess */
				);
				$foo = path('XMLFILE_PARAMETERS', preg_replace($pattern, $folderName, path('XMLFILE_PARAMETERS')));
				// $plxAdmin->aConf n'est pas mis à jour (passage de paramètre par valeur)
				$plxAdmin->editConfiguration($plxAdmin->aConf, $newConfig);

				return true;
			} else {
				plxMsg::Error(sprintf($this->getLang('UNWRITABLE_FOLDER'), $folderName));
			}
		} else {
			plxMsg::Error(sprintf($this->getLang('FOLDER_EXISTS'), $folderName));
		}
		return false;
	}

	/**
	 * Met à jour le fichier config.php en fonction du nom du dossier de données.
	 * */
	private function __configUpdate($folder) {
		// On vérifie qu'on ne modifie pas config.php pour utiliser le dossier de données actuel.
		if(!preg_match('@^'.$folder.'/@', PLX_CONFIG_PATH)) {
			$content = file_get_contents($this->config_filename);
			$newContent = preg_replace(self::CONFIG_MASK, '$1' . $folder, $content);
			if(empty($newContent)) {
				plxMsg::Error('$newContent is empty');
				return;
			}
			plxUtils::debugJS('folder', $folder);
			plxUtils::debugJS('content', $content);
			plxUtils::debugJS('newContent', $newContent);
			$backup_filename = $this->config_filename . '.bak';
			# Token pour rediriger vers la page d'accueil côté site
			file_put_contents(self::TOKEN_FILE, PLX_CONFIG_PATH);
			try {
				copy($this->config_filename, $backup_filename);
				file_put_contents($this->config_filename, $newContent, LOCK_EX);

				// forces core/admin/medias.php to init the following variable
				unset($_SESSION['medias']);

				// clearstatcache(true);
				if (function_exists('opcache_invalidate')) {
					opcache_invalidate($this->config_filename);
				}
			} catch(Exception $err) {
				plxMsg::Error($err->getMessage());
			}
		} else {
			plxMsg::Error($this->getLang('THIS_IS_THE_CURRENT_FOLDER'));
		}
	}

	/**.
	 * Traite l'envoi du formulaire pour changer le dossier de données.
	 * */
	public function process($oldName, $newName) {
		global $plxAdmin;

		if(!is_writable($this->config_filename)) {
			plxMsg::Error(sprintf($this->getLang('UNWRITABLE_FILE'), 'config.php'));
			return false;
		}

		$dataFolders = $this->getDatasFolders();
		// On vérifie que $newName est valide et qu'il n'existe pas déjà un dossier $newName
		if(!empty($newName) and (
				!preg_match('@^([^/]+)/?$@', $newName, $matches) or
				in_array($newName, $dataFolders)
			)
		) {
			plxMsg::Error(sprintf($this->getLang('INVALIDATE_NEW_NAME'), $newName));
			return false;
		}

		// Normalisation de $newName. Doit finir par '/'.
		if(!empty($matches)) { $newName = $matches[1] . '/'; }

		if(!empty($oldName)) {
			// On valide $oldName
			if(in_array($oldName, $dataFolders)) {
				if(empty($newName)) {
					// Changement de dossier pour les données
					$this->__configUpdate($oldName);
				} else {
					// Renommage du dossier pour les données
					if($this->__renameDatasFolder($oldName, $newName)) {
						$this->__configUpdate($newName);
					}
				}
			} else {
				// valeurs invalide pour $oldName
				plxMsg::Error(sprintf($this->getLang('INVALIDATE_OLD_NAME'), $oldName));
				return false;
			}
		} else {
			// Nouveau dossier
			if($this->__createDatasFolder($newName)) {
				$this->__configUpdate($newName);
			}
		}

		return true;
	}

	/* =============== admin Hooks ============= */

	/*
	 * Si les fichiers config.php et xxx/configuration/parametres.xml ont des dates différentes,
	 * on force la réécriture de .htaccess et de plugins/admin.css et de plugins/site.css.
	 * Le fichier .htaccess avec les redirections d'urls est généré dans tous les cas, même si urlrewrite est désactivé.
	 * */
	public function plxAdminConstruct() {
		echo self::BEGIN_CODE;
?>
$kzTimestamp = filemtime(path('XMLFILE_PARAMETERS'));
$kzFilename = PLX_ROOT . 'config.php';
if(filemtime($kzFilename) != $kzTimestamp) {
	if($this->htaccess('1', preg_replace('@/core/admin/[\w-]+\.php$@', '/', $_SERVER['SCRIPT_NAME']))) {
		touch($kzFilename, $kzTimestamp);
		foreach(array('admin', 'site') as $type) {
			$this->plxPlugins->cssCache($type);
		}
	} else {
		plxUtils::Error(L_UPDATE_ERR_UPDATE_HTACCESS_FILE);
	}
}
<?php
		echo self::END_CODE;
	}

	/* =============== site Hooks ============= */

	public function Index() {
		if(file_exists(self::TOKEN_FILE)) {
			$content = trim(file_get_contents(self::TOKEN_FILE));
			unlink(self::TOKEN_FILE);
			if($content != PLX_CONFIG_PATH) {
				# Force le retour à la page d'accueil en cas de changement de dossier de données
				header('Location: ' . plxUtils::getRacine() . 'index.php');
				exit;
			}
		}
	}

	/* ------------ Hooks non utilisés ------------ */
/*
	public function AdminPrepend() {

		# durée minimum pour que le cache de PHP se mette à jour - Pb avec include('config.php')
		const MMD_RELAX_TIME = 5;

	# hack against cache include file in PHP
		$referer = basename($_SERVER['HTTP_REFERER'], '.php');
		$p = strrpos($referer, '?p='.__CLASS__);
		if (true or ($p > 0)) {
			$content = file_get_contents(PLX_ROOT.'config.php');
			$str = preg_replace('#^.*\'PLX_CONFIG_PATH\'[^\']*\'([^\']*)\'.*$#s', '$1', $content);
			if ($str != PLX_CONFIG_PATH) {
				if (self::MMD_RELAX_TIME > 0) {
					header('Refresh: '.self::MMD_RELAX_TIME);
				}
				$this->config = $str;
			}
		}
	}

	public function plxAdminConstruct() {
		$code = <<< 'CODE'
<?php
	$plugin = $this->plxPlugins->aPlugins['##CLASS##'];
	$plugin->url = $this->racine.$this->aConfig['racine_plugins'].'##CLASS##/';
	$plugin->currentUser = $this->aUsers[$_SESSION['user']];
?>
CODE;
		$params = array(
			'##CLASS##'	=> __CLASS__
		);
		echo str_replace(array_keys($params), array_values($params), $code);
	}
 * */

}
